/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.

  Some parts of this code are derived from ITK. See ITKCopyright.txt
  for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbMirrorBoundaryCondition_h
#define __otbMirrorBoundaryCondition_h
#include "itkNeighborhood.h"
#include "itkImageBoundaryCondition.h"

namespace otb
{

/** \class MirrorBoundaryCondition
 * \brief
 * A function object that determines values outside of image boundaries
 * according to mirror conditions.
 *
 * The input to this function object is a neighborhood iterator.  This boundary
 * condition object is designed to be given as a template argument to a
 * NeighborhoodIterator or any of the NeighborhoodIterator subclasses.
 *
 * \ingroup DataRepresentation
 * \ingroup ImageObjects
 */
template<class TImage>
class ITK_EXPORT MirrorBoundaryCondition
  : public itk::ImageBoundaryCondition<TImage>
{
public:
  /** Standard class typedefs. */
  typedef MirrorBoundaryCondition             Self;
  typedef itk::ImageBoundaryCondition<TImage> Superclass;

  /** Extract information from the image type. */
  typedef typename Superclass::PixelType        PixelType;
  typedef typename Superclass::PixelPointerType PixelPointerType;
  typedef typename Superclass::IndexType        IndexType;
  typedef typename Superclass::OffsetType       OffsetType;
  typedef typename Superclass::NeighborhoodType NeighborhoodType;

  typedef typename Superclass::NeighborhoodAccessorFunctorType
  NeighborhoodAccessorFunctorType;

  /** Extract information from the image type. */
  itkStaticConstMacro(ImageDimension, unsigned int, Superclass::ImageDimension);

  /** Default constructor. */
  MirrorBoundaryCondition() {}

  /** Computes and returns a neighborhood of appropriate values from
   * neighborhood iterator data.. */
  virtual PixelType operator ()(const OffsetType& point_index,
                                const OffsetType& boundary_offset,
                                const NeighborhoodType *data) const;

  /** Computes and returns the appropriate pixel value from
   * neighborhood iterator data, using the functor. */
  virtual PixelType operator ()(
    const OffsetType& point_index,
    const OffsetType& boundary_offset,
    const NeighborhoodType *data,
    const NeighborhoodAccessorFunctorType& neighborhoodAccessorFunctor) const;
};

} // end namespace itk

#ifndef OTB_MANUAL_INSTANTIATION
# include "otbMirrorBoundaryCondition.txx"
#endif

#endif
